import re
import traceback

from selenium import webdriver # webdriver 操作一般用
from selenium.webdriver.chrome import service as fs # Chrome を driver として設定する用
from selenium.webdriver.chrome.options import Options # headless モードで作業する用
from bs4 import BeautifulSoup


class Regular():
    """This class gets URLs and retrieve the body text from them 
    with NEWS WEB (regular news site, not the Easy Japanese one) solely.

    Most of methods you'll call on Easy objects are using Selenium 
    or Beautiful Soup.

    This class has 2 roles. 
    1. Access the NEWS WEB and get the list of URLS of the most recent day.
    2. Retrieve the body text of these URLs.

    These methods serves for the 1st role.
      * start(url, url_or_article)
      * shutdown()
      * get_url(soup)

    These methods serves for the 2nd role.
      * start(url, url_or_article)
      * shutdown()
      * get_article(soup)

    Parameters
    ----------
    DRIVER_PATH : str
        The filepath of your Chrome driver. To prevent the tedious work of inputting,
        get this path from the attribute of Easy() class.

    Attributes
    ----------
    DRIVER_PATH : str
        The filepath of your Chrome driver.
    service : selenium.webdriver.chrome.service.Service
        The setting for starting up a browser.
    options : selenium.webdriver.chrome.options.Options
        The setting for starting up a browser.

    driver : selenium.webdriver.chrome.webdriver.WebDriver
        The web driver you use for the all process.
    html_url : bytes
        The web page data encoded to HTML for the URL retrieval.
    soup_url : bs4.BeautifulSoup
        The parsed web page data for the URL retrieval.
    html_article : bytes
        The web page data encoded to HTML for the article retrieval.
    soup_article : bs4.BeautifulSoup
        The parsed web page data for the article retrieval.
    """
    def __init__(self, DRIVER_PATH):
        """Constructor.
        Define the attributes for start().
        """
        self.DRIVER_PATH = DRIVER_PATH
        self.service = fs.Service(executable_path=self.DRIVER_PATH)
        self.options = Options()
        self.options.add_argument('--window-size=1920,1200')
        self.options.add_argument('--headless')


    def start(self, url, url_or_article=True):
        self.driver = webdriver.Chrome(options=self.options, service=self.service)
        """Open a browser, parse the JavaScript data and save it as HTML data.
        
        Parameters
        ----------
        url : str
            The web page you want to open.
        url_or_article : bool
            Whether you want to start retrieving URL or not.
        """
        self.driver.get(url)
        print('Done starting up a new browser!')
        html = self.driver.page_source.encode('utf-8')
        print('Done parsing the JavaScript data!')
        soup = BeautifulSoup(html, 'html.parser')
        print('Done reading the data as HTML!')
        if url_or_article:
            self.html_url = html 
            self.soup_url = soup
        else:
            self.html_article = html
            self.soup_article = soup

        print('All done start_url(). Hello!')


    def shutdown(self):
        """Shut down the driver.
        """
        self.driver.quit()
        print('All done shutdown(). Goodbye!')


    def get_url(self, soup):
        """Extract the URL from the parsed HTML data.

        Parameters
        ----------
        soup : bs4.BeautifulSoup
            The parsed HTML data.
        
        """
        try:
            reg_url = soup.select_one('#js-regular-news').get('href')
            print('Done retrieving the URL!')
        except:
            print('An unexpected error has occured during obtaining the URL.')
            traceback.print_exc()
            reg_url = 'Unexpected'
        return reg_url
    
    # soup := each element of self.soup_article
    def get_article(self, soup):
        """Retrieve the body text of the news.

        Parameters
        ----------
        soup : bs4.BeautifulSoup
            The parsed HTML data.
        """
        try:
            for nav in soup(['nav']):
                nav.decompose()
            for title in soup(['h2']):
                title.decompose()
            for fig in soup(['figcaption']):
                fig.decompose()
            for editor in soup.findAll(class_='content--editor-body'):
                editor.decompose()
            print('Done removing the noise!')

        except:
            print('An expected error has occured during editing the html data.')
            traceback.print_exc()
            soup = None

        try:
            # this shoud be it if there happened an error in the extraction process.
            article = soup.find(class_='content--detail-body').text
            article = re.sub('\n', '', article)
            article = re.sub(' ', '', article)
            reg_article = article

        except:
            print('An expected error has occured during extracting the text.')
            traceback.print_exc()
            reg_article = 'Unexpected'
        return reg_article


import traceback
import datetime

import pandas as pd


class Complete():
    """This class completes the whole process of web scraping.

    So far, I define the methods for obtaining the date of the retrieved easy 
    news data and creating a dataframe from the retireved datas. If there is
    a need for another manipulations related to this whole scraping processes,
    then a code for the manipulations will be added here.

    Attributes
    ----------
    easy_urls : list
        A list of the NEWS WEB EASY URLs.
    easy_articles : list
        A list of the body text of NEWS WEB EASY articles.
    regular_urls : list
        A list of the NEWS WEB URLs.
    regular_articles : list
        A list of the body text of NEWS WEB articles.
    date : str
        Date of the newly retrieved data
    SAVE_PATH : str
        The filepath of the directory where you want to save all the retrieved data.
    df_old : pandas.core.frame.DataFrame
        A pandas dataframe that you want to concatenate your new data with.
    df : pandas.core.frame.DataFrame
        The pandas dataframe that you make from the retrieved data.
    df_piled : pandas.core.frame.DataFrame
        A pandas dataframe that you concatenate df with df_old.
    date_new : datetime.datetime
        The date of the newly retrieved data.
    """
    def __init__(self, easy_urls, easy_articles,
                regular_urls, regular_articles, date):
        """Constructor.
        Define the attributes mainly for creating the data frame.
        """
        self.easy_urls = easy_urls
        self.easy_articles = easy_articles
        self.regular_urls = regular_urls
        self.regular_articles = regular_articles
        self.date = date
        SAVE_PATH = input('The filepath, where you want to save the final output data?')
        self.SAVE_PATH = SAVE_PATH
        self.df_old = pd.read_csv(self.SAVE_PATH)
        

    def make_df(self):
        """Create the dataframe from the retrieved data.
        """
        self.input_length_test()
        new_date = self.get_date()
        self.df = pd.DataFrame({
            'Date': new_date,
            'Easy URL': self.easy_urls,
            'Easy article': self.easy_articles,
            'Regular URL': self.regular_urls,
            'Regular article': self.regular_articles
        })
        
        
    def concat(self):
        """Pile up the created dataframe on the existing dataframe and save it.
        """
        self.df_piled = pd.concat([self.df, self.df_old])
        self.df_piled['Date'] = pd.to_datetime(self.df_piled['Date'])
        self.df_piled.to_csv(self.SAVE_PATH, index=False)
        return self.df_piled
    

    
    def input_length_test(self):
        """(Soon to be deleted) Check if the retrieved datas have unequal shape.
        """
        try:
            if ~(
                len(self.easy_urls) == len(self.easy_articles)\
                == len(self.regular_urls) == len(self.regular_articles)
            ):
                raise LengthError('An unexpected error has occured with some or all of the four inputs.')
        except LengthError:
            print(traceback.print_exc())

    def get_date(self):
        """Obtain the date of newly retrieded data.
        """
        old_date = datetime.datetime.strptime(
            self.df_old['Date'][0], '%Y-%m-%d'
            )
        new_date_raw = datetime.datetime.strptime(
                self.date, '%m月%d日'
                )
        if (old_date.month==12) & (new_date_raw.month==1):
            new_year = old_date.year + 1
        else:
            new_year = old_date.year
        self.date_new = datetime.datetime(
                new_year, new_date_raw.month, new_date_raw.day
                )
        return self.date_new



class LengthError(Exception):
    """This error notifies the unequal length of the inputs; URLs and articles.
    """
    pass


"""Let's study Easy Japanese (やさしいにほんご) daily with NHK news web easy!

This module (package) mainly covers the topics listed below.
- To get the URLs of the easy Japanese news articles (easy.py)
- To extract the body texts of them (easy.py)
- To get the URL of the original articles of them (regular.py)
- To extract the original ones as well (regular.py)
- To concatenate all the above data (complete.py)
- To save them all as a csv file (complete.py)

I do not own any rights for the articles, nor take any responsibility caused 
by the use of this module. 

Have a great Easy Japanese (やさしいにほんご) life!
"""


import sys
import time

from tqdm import tqdm

# I don't know but maybe you don't need to import these? 
# I'd really appreciate if somebody knows the answer! 
from easyjapanese.easy import Easy
from easyjapanese.regular import Regular
from easyjapanese.complete import Complete

# The very first thing we do is give a useful error if someone is
# running this code under Python 2.
if sys.version_info.major < 3:
    raise ImportError('You are trying to use a Python 3-specific version of Easy Japanese under Python 2. This will not work. Please do something on it lol')

# I don't know what to include. Should I include the other libraries (e.g. sys, bs4.BeautifulSoup, ...)
__all__ = ['EasyJapanese', 'Easy', 'Regular', 'Complete']
__version__ = '0.0.2'



def EasyJapanese():
    """Run all the modules and return a dataframe that includes the newly retrieved data.
    """
    easy_urls = []
    easy_articles = []
    soups_easy = []
    regular_urls = []
    regular_articles = []
    soups_regular_url = []
    soups_regular_article = []

    e = Easy()
    # easy_urls
    e.start_url()
    e.get_raw_url_date()
    for url in tqdm(e.easy_urls_raw):
        easy_url = e.get_url(url)
        easy_urls.append(easy_url)
    print('Done retrieving the URLs!')
    print('All done with easy_urls! Great job!')
    e.shutdown()

    # easy_articles
    for url in tqdm(easy_urls):
        soup = e.start_extract(url)
        soups_easy.append(soup)
        time.sleep(1)
    for soup in tqdm(soups_easy):
        easy_article = e.get_article(soup)
        easy_articles.append(easy_article)
    print('Done retrieving the articles!')
    print('All done with easy_articles! Great job!')
    e.shutdown()

    # new_date
    new_date = e.date

    r = Regular(e.DRIVER_PATH)
    # regular_urls
    for url in tqdm(easy_urls):
        r.start(url, url_or_article=True)
        soups_regular_url.append(r.soup_url)
        time.sleep(1)
    for soup in tqdm(soups_regular_url):
        reg_url = r.get_url(soup)
        regular_urls.append(reg_url)
    print('Done retrieving the URLs!')
    print('All done with regular_urls! Great job!')
    r.shutdown()

    # regular_articles
    for url in tqdm(regular_urls):
        r.start(url, url_or_article=False)
        soups_regular_article.append(r.soup_article)
        time.sleep(1)
    for article in tqdm(soups_regular_article):
        reg_article = r.get_article(article)
        regular_articles.append(reg_article)
    print('Done retrieving the articles!')
    print('All done with regular_articles! Great job!')
    r.shutdown()



    c = Complete(
            easy_urls, easy_articles,
            regular_urls, regular_articles,
            new_date
            )
    c.make_df()
    df_new = c.concat()
    return df_new



from setuptools import setup
import easyjapanese

description = 'Web scraping tool for NHK web news easy'
NAME = 'easyjapanese'
AUTHOR = 'Yuki Cossy'
AUTHOR_EMAIL = 'domokomod5@gmail.com'
URL = 'https://github.com/yuki-cossy/EasyJapanese'
LICENSE = 'MIT'
DOWNLOAD_URL = 'https://github.com/yuki-cossy/EasyJapanese'
VERSION = easyjapanese.__version__
PYTHON_REQUIRES = ">=3.6"
INSTALL_REQUIRES = ['tqdm', 'pandas', 'selenium', 'beautifulsoup4']
PACKAGES = ['easyjapanese']

# I don't know why but it didn't work out in the way I wanted.
with open('README4PyPI.md', 'r') as desc:
    long_description = desc.read()


setup(name=NAME,
      author=AUTHOR,
      author_email=AUTHOR_EMAIL,
      maintainer=AUTHOR,
      maintainer_email=AUTHOR_EMAIL,
      description=description,
      long_description=long_description,
      license=LICENSE,
      url=URL,
      version=VERSION,
      download_url=DOWNLOAD_URL,
      python_requires=PYTHON_REQUIRES,
      install_requires=INSTALL_REQUIRES,
      packages=PACKAGES,
    )


import re
import traceback

from selenium import webdriver # webdriver 操作一般用
from selenium.webdriver.chrome import service as fs # Chrome を driver として設定する用
from selenium.webdriver.chrome.options import Options # headless モードで作業する用
from selenium.webdriver.common.by import By # find_element() で参照したい位置を特定する用
from bs4 import BeautifulSoup

class Easy():
    """This class gets URLs and retrieve the body text from them 
    with NEWS WEB EASY solely.

    Most of methods you'll call on Easy objects are using Selenium 
    or Beautiful Soup.
    
    This class has 2 roles. 
    1. Access the NEWS WEB EASY and get the list of URLS of the most recent day.
    2. Retrieve the body text of these URLs.
    
    These methods serves for the 1st role.
      * start_url()
      * shutdown()
      * get_raw_url_date()
      * get_url(url)

    These methods serves for the 2nd role.
      * start_extract(url)
      * shutdown()
      * get_article(soup)

    if you wish to retrieve the information from the past article, 
    then you can do so by modifying the `start_url()` method, specifically,
    the XPATH of element.

    Attributes
    ----------
    DRIVER_PATH : str
        The filepath of your Chrome driver.
    service : selenium.webdriver.chrome.service.Service
        The setting for starting up a browser.
    options : selenium.webdriver.chrome.options.Options
        The setting for starting up a browser.
    BASE_URL : str
        The web page you start off with when you extract the URL info.
    driver : selenium.webdriver.chrome.webdriver.WebDriver
        The web driver you use for the all process.
    html_url : bytes
        The web page data encoded to HTML for the URL retrieval.
    soup_url : bs4.BeautifulSoup
        The parsed web page data for the URL retrieval.
    html_article : bytes
        The web page data encoded to HTML for the article retrieval.
    soup_article : bs4.BeautifulSoup
        The parsed web page data for the article retrieval.
    easy_urls_raw : bs4.element.ResultSet
        The elements you find and clip from the HTML data.
    date : str
        The date of the retrieved data.
    """
    def __init__(self):
        """Constructor.
        Define the attributes for start_url(), start_extract(), and get_url().
        """
        DRIVER_PATH = input('The filepath, where your Chrome Driver is?')
        self.DRIVER_PATH = DRIVER_PATH
        self.service = fs.Service(executable_path=self.DRIVER_PATH)
        self.options = Options()
        self.options.add_argument('--window-size=1920,1200')
        self.options.add_argument('--headless')
        self.BASE_URL = 'https://www3.nhk.or.jp/news/easy'

        
    def start_url(self):
        """Open the NEWS WEB EASY, display the list of URLS and parse 
        and pass it to Beautiful Soup as HTML.
        
        *** I save html_url and html_article as attribute solely for the debug purpose.
        """    
        self.driver = webdriver.Chrome(options=self.options, service=self.service)
        self.driver.get(self.BASE_URL)
        print('Done starting up a new browser!')

        self.driver.execute_script('window.scrollBy(0, 1200);')
        element = self.driver.find_element(
                By.XPATH, '//*[@id="easy-wrapper"]/div[2]/aside/section[2]/div[1]/a[1]'
                )
        element.click()
        print('Done displyaing the list of the URLs!')

        self.html_url = self.driver.page_source.encode('utf-8')
        print('Done parsing the JavaScript data!')
        self.soup_url = BeautifulSoup(self.html_url, 'html.parser')
        print('Done reading the data as HTML!')

        print('All done start_url(). Hello!')


    def start_extract(self, url):
        """Open the article and parse it to HTML.
        
        Parameters
        ----------
        url : str
            The URL you want to open.
        """
        self.driver = webdriver.Chrome(options=self.options, service=self.service)
        self.driver.get(url)
        print('Done starting up a new browswer!')

        self.html_article = self.driver.page_source.encode('utf-8')
        print('Done parsing the JavaScript data!')
        self.soup_article = BeautifulSoup(self.html_article, 'html.parser')
        print('Done reading the data as HTML!')

        print('All done start_extract(). Hello!')
        return self.soup_article


    def shutdown(self):
        """Shut down the webdriver."""
        self.driver.quit()
        print('All done shutdown(). Goodbye!')

    def get_raw_url_date(self):
        """Extract the URLs from the parsed HTML data.
        
        *** I separate this method from get_url() just because I don't
        want to include this in for-loop done in EasyJapanese() (in
        __init__.py).
        """
        self.easy_urls_raw = self.soup_url.select('#js-archives-list > a')
        self.date = self.soup_url.select_one('#js-pager-date').text
        print('Done getting the raw text of URLs!')


    def get_url(self, url):
        """Retrieve the URL from the get_raw_url_date() data.
        
        Parameters
        ----------
        url : str
            The parsed and clipped HTML data (bs4.element.Tag). You're supposed
            to create bs4.element.Tag input out of bs4.element.ResultSet.
        """
        try:
            raw_url = url.get('href')
            print('Done extracting the raw URL!')
            easy_url = re.sub(r'^./', self.BASE_URL+'/', raw_url)
            print('Done retrieving the URL!')
        except:
            print('An unexpected error has occured during obtaining the URL.')
            traceback.print_exc()
            easy_url = 'Unexpected'
        return easy_url


    def get_article(self, soup):
        """Retrieve the body text from the parsed HTML data.
        
        Parameters
        ----------
        soup : bs4.BeautifulSoup
            The parsed HTML data.
        """
        try:
            for ruby in soup(['rt']):
                ruby.decompose()
            print('Done removing the noise!')
            article_raw = soup.select_one('#js-article-body').text
            article_raw = re.sub('\n', '', article_raw)
            easy_article = re.sub(' ', '', article_raw) 

        except:
            print('An unexpected error has occured during obtaining the article.')
            print(f'soup: {soup}\n')
            traceback.print_exc()
            easy_article = 'Unexpected'
        return easy_article
    
